import time

from .config_type import EConfigType
from .xray_config import XrayConfig, BaseConfig


class ServerConfig(BaseConfig):
    def __init__(self, config_version=3, config_type=None, added_time=0, remarks="", outbound_bean=None,
                 full_config=None):
        self.configVersion = config_version
        self.configType = config_type
        self.addedTime = added_time if added_time != 0 else int(time.time() * 1000)
        self.remarks = remarks
        self.outboundBean = outbound_bean
        self.fullConfig = full_config

    @staticmethod
    def create(config_type_tuple):
        config_type = config_type_tuple.value[1]
        protocol = config_type_tuple.value[0]
        if (config_type == EConfigType.VMESS.value[1] or
                config_type == EConfigType.VLESS.value[1]):
            return ServerConfig(
                config_type=config_type,
                outbound_bean=XrayConfig.OutboundBean(
                    protocol=protocol.lower(),
                    settings=XrayConfig.OutboundBean.OutSettingsBean(
                        vnext=[XrayConfig.OutboundBean.OutSettingsBean.VnextBean(
                            users=[XrayConfig.OutboundBean.OutSettingsBean.VnextBean.UsersBean()]
                        )]
                    ),
                    stream_settings=XrayConfig.OutboundBean.StreamSettingsBean()
                )
            )

        elif (config_type == EConfigType.CUSTOM.value[1] or
              config_type == EConfigType.WIREGUARD.value[1]):
            return ServerConfig(config_type=config_type)

        elif (config_type == EConfigType.SHADOWSOCKS.value[1] or
              config_type == EConfigType.SOCKS.value[1] or
              config_type == EConfigType.TROJAN.value[1]):
            return ServerConfig(
                config_type=config_type,
                outbound_bean=XrayConfig.OutboundBean(
                    protocol=protocol.lower(),
                    settings=XrayConfig.OutboundBean.OutSettingsBean(
                        servers=[XrayConfig.OutboundBean.OutSettingsBean.ServersBean()]
                    ),
                    stream_settings=XrayConfig.OutboundBean.StreamSettingsBean()
                )
            )

    def get_proxy_outbound(self):
        if self.configType != EConfigType.CUSTOM.value[1]:
            return self.outboundBean
        return self.fullConfig.get_proxy_outbound() if self.fullConfig else None



import dataclasses
from abc import ABC, abstractmethod
from typing import Optional
from urllib.parse import urlparse, parse_qs, quote
import urllib.parse as up

from xray_protocol_helper.base64_helper import Base64Helper
from xray_protocol_helper.xray_json_creator.xray_config_util import XrayConfigUtil


@dataclasses.dataclass
class XrayDecodeData:
    uri: str
    protocol: str
    address: str
    port: int
    sni: str = None
    host: str = None

    def __post_init__(self):
        if self.sni and "*." in self.sni:
            self.sni = self.sni.replace("*.", "")
        if self.host and "*." in self.host:
            self.host = self.host.replace("*.", "")


class AbstractXrayProtocol(ABC):
    REPLACE_DATA_LIST = ("\'", "\"", "`")

    def __init__(self, uri: str = None, config: dict = None):
        self.uri = uri
        self.config = config

    @property
    def pars_result(self):
        return urlparse(self.uri)

    @property
    def uri_body(self):
        return self.uri.split("://")[1]

    @property
    def dict_uri(self) -> dict:
        return Base64Helper.decode_dict_base64(encoded_dict_data=self.uri_body)

    @classmethod
    def clean_uri_string(cls, uri):
        uri = uri.strip()
        for replace_char in cls.REPLACE_DATA_LIST:
            uri = uri.replace(replace_char, "")
        uri = uri.split("#")[0]
        return uri

    def get_data(self) -> Optional[XrayDecodeData]:
        if self.uri is None and self.config:
            self.uri = self._get_uri_from_config()
        self._clean_uri()
        self._validate_uri()
        return self._get_data_from_uri()

    @abstractmethod
    def _get_uri_from_config(self) -> str:  # pragma: no cover
        pass

    @abstractmethod
    def _get_data_from_uri(self) -> Optional[XrayDecodeData]:  # pragma: no cover
        pass

    def _clean_uri(self):
        self.uri = self.clean_uri_string(uri=self.uri)

    def _validate_uri(self):
        result = XrayConfigUtil().get_xray_config(uri=self.uri, port=1080)
        if not result.status:
            raise ValueError("uri is not valid")


class VmessProtocol(AbstractXrayProtocol):
    def _get_data_from_uri(self) -> Optional[XrayDecodeData]:
        dict_url = self.dict_uri
        if not dict_url:
            return  # pragma: no cover
        address = dict_url.get("add")
        port = dict_url.get("port")
        sni = dict_url.get("sni")
        sni = None if sni == "" else sni
        host = dict_url.get("host")
        host = None if host == "" else host
        return XrayDecodeData(uri=self.uri, protocol="vmess", address=address, port=port, sni=sni, host=host)

    def _clean_uri(self):
        super()._clean_uri()
        dict_uri = self.dict_uri
        if not dict_uri:
            return
        dict_uri["ps"] = ""
        dict_uri = dict(sorted(dict_uri.items()))
        self.uri = f"vmess://{Base64Helper.encode_dict_base64(dict_data=dict_uri)}"

    def _get_uri_from_config(self) -> str:
        network_protocol = self._get_network_protocol()
        config_dict = self._call_network_protocol_func(network_protocol)
        return f"vmess://{Base64Helper.encode_dict_base64(config_dict)}"

    def _get_network_protocol(self):
        return self.config.get("outbounds", [{}])[0].get("streamSettings", {}).get("network", None)

    def _call_network_protocol_func(self, network_protocol):
        network_protocol_functions = {
            "ws": self.vmess_dict_ws_network,
            "grpc": self.vmess_dict_grpc_network,
            "tcp": self.vmess_dict_tcp_network,
        }
        network_protocol_func = network_protocol_functions.get(network_protocol)
        if network_protocol_func is None:
            raise ValueError("not found protocol")
        return network_protocol_func()

    @dataclasses.dataclass
    class CommonVmessData:
        config: dict

        def __post_init__(self):
            self.outbound_setting = self.config.get("outbounds", [{}])[0]
            self.stream_settings = self.outbound_setting.get("streamSettings", {})
            self.vnext_setting = self.outbound_setting.get("settings", {}).get("vnext", [{}])[0]
            self.address = self.vnext_setting.get("address", "")
            self.port = self.vnext_setting.get("port", "")
            if not self.address or not self.port:
                raise ValueError("address or port not found")
            self.id = self.vnext_setting.get("users", [{}])[0].get("id", "")
            self.alter_id = self.vnext_setting.get("users", [{}])[0].get("alterId", 64)

    def vmess_dict_ws_network(self):
        common_vmess_data = self.CommonVmessData(config=self.config)
        return {
            "ps": "",
            "add": common_vmess_data.address,
            "port": common_vmess_data.port,
            "tls": common_vmess_data.stream_settings.get("security", ""),
            "id": common_vmess_data.id,
            "aid": common_vmess_data.alter_id,

            "host": common_vmess_data.stream_settings.get("wsSettings", {}).get("headers", {}).get("Host", ""),
            "net": "ws",
            "path": common_vmess_data.stream_settings.get("wsSettings", {}).get("path", ""),
            "type": common_vmess_data.vnext_setting.get("users", [{}])[0].get("security", ""),
            "sni": common_vmess_data.stream_settings.get("tlsSettings", {}).get("serverName", "")
        }

    def vmess_dict_grpc_network(self):
        common_vmess_data = self.CommonVmessData(config=self.config)
        return {
            "v": "2",
            "ps": "",
            "add": common_vmess_data.vnext_setting.get("address", ""),
            "port": common_vmess_data.vnext_setting.get("port", ""),
            "id": common_vmess_data.id,
            "aid": common_vmess_data.alter_id,
            "net": "grpc",
            "type": "none",
            "host": "",
            "path": common_vmess_data.stream_settings.get("grpcSettings", {}).get("serviceName", ""),
            "tls": common_vmess_data.stream_settings.get("security", ""),
            "sni": common_vmess_data.stream_settings.get("tlsSettings", {}).get("serverName", ""),
            "scy": common_vmess_data.vnext_setting.get("users", [{}])[0].get("security", "")
        }

    def vmess_dict_tcp_network(self):
        common_vmess_data = self.CommonVmessData(config=self.config)
        return {
            "v": "2",
            "ps": "",
            "add": common_vmess_data.vnext_setting.get("address", ""),
            "port": common_vmess_data.vnext_setting.get("port", ""),
            "id": common_vmess_data.id,
            "aid": common_vmess_data.alter_id,
            "net": "tcp",
            "type": common_vmess_data.stream_settings.get("tcpSettings", {}).get("header", {}).get("type", ""),
            "host": common_vmess_data.stream_settings.get("tcpSettings", {}).get("header", {}).get("request", {})
                                                                            .get("headers", {}).get("Host", [])[0],
            "path": common_vmess_data.stream_settings.get("tcpSettings", {}).get("header", {}).get("request", {}).get(
                "path", [])[0],
            "tls": common_vmess_data.stream_settings.get("security", "")

        }


class VlessProtocol(AbstractXrayProtocol):
    PROTOCOL = "vless"

    def _get_data_from_uri(self) -> Optional[XrayDecodeData]:
        return self.get_vless_data()

    def get_vless_data(self, pars_result=None) -> Optional[XrayDecodeData]:
        if pars_result is None:
            pars_result = self.pars_result
        address = pars_result.hostname
        port = pars_result.port
        pars_query = parse_qs(pars_result.query)
        try:
            host = pars_query["host"][0]
            host = None if host == "" else host
        except KeyError:
            host = None
        try:
            sni = pars_query["sni"][0]
            sni = None if sni == "" else sni
        except KeyError:
            sni = None
        return XrayDecodeData(uri=self.uri, protocol=self.PROTOCOL, address=address, port=port, sni=sni, host=host)

    def _get_uri_from_config(self) -> str:
        network_protocol = self._get_network_protocol()
        return self._call_network_protocol_func(network_protocol)

    def _get_network_protocol(self):
        return self.config.get("outbounds", [{}])[0].get("streamSettings", {}).get("network", None)

    def _call_network_protocol_func(self, network_protocol):
        network_protocol_functions = {
            "ws": self.vless_uri_ws_network,
            "grpc": self.vless_uri_grpc_network,
            "tcp": self.vless_uri_tcp_network,
            "xhttp": self.vless_uri_xhttp_network,
            "httpupgrade": self.vless_uri_httpupgrade_network
        }
        network_protocol_func = network_protocol_functions.get(network_protocol)
        if network_protocol_func is None:
            raise ValueError("not found protocol")
        return network_protocol_func()

    @dataclasses.dataclass
    class CommonVlessData:
        config: dict

        def __post_init__(self):
            self.outbound_setting = self.config.get("outbounds", [{}])[0]
            self.stream_settings = self.outbound_setting.get("streamSettings", {})
            self.vnext_setting: dict = self.outbound_setting.get("settings", {}).get("vnext", [{}])[0]
            self.address = self.vnext_setting.get("address")
            self.port = self.vnext_setting.get("port")
            if not self.address or not self.port:
                raise ValueError("address or port not found")
            self.vless_settings = self.vnext_setting.get("users", [{}])[0]
            self.id = self.vless_settings.get("id")
            self.encryption = self.vless_settings.get("encryption", "")
            self.flow = self.vless_settings.get("flow", "")
            self.security = self.stream_settings.get("security", "")
            self.network_type = self.stream_settings.get("network", "")
            self.tlsSettings = self.stream_settings.get("tlsSettings", {})
            self.realitySettings = self.stream_settings.get("realitySettings", {})

    def security_fields(self, common_vless_data: CommonVlessData) -> str:
        security = common_vless_data.security.lower()
        if security == "none" or security == "":
            return ""
        elif security == "tls":
            return f"&sni={common_vless_data.tlsSettings.get("serverName", "")}"
        elif security == "reality":
            return f"&sni={common_vless_data.realitySettings.get('serverName', '')}&pbk={common_vless_data.realitySettings.get('publicKey', '')}&sid={common_vless_data.realitySettings.get('shortId', '')}"

    def vless_uri_ws_network(self):
        common_vless_data = self.CommonVlessData(config=self.config)
        path = quote(common_vless_data.stream_settings.get("wsSettings", {}).get("path", ""), safe="")
        vless_host = common_vless_data.stream_settings.get("wsSettings", {}).get("headers", {}).get("Host", "")
        vless_host = vless_host.replace(" ", "").strip()
        return f"vless://{common_vless_data.id}@{common_vless_data.address}:{common_vless_data.port}?" \
               f"path={path}&" \
               f"security={common_vless_data.security}&" \
               f"encryption={common_vless_data.encryption}&" \
               f"flow={common_vless_data.flow}&" \
               f"host={vless_host}&" \
               f"type={common_vless_data.network_type}" \
               f"{self.security_fields(common_vless_data)}"

    def vless_uri_tcp_network(self) -> str:
        common_vless_data = self.CommonVlessData(config=self.config)
        header_type = common_vless_data.stream_settings.get("tcpSettings", {}).get("header", {}).get("type", "none")
        return f"vless://{common_vless_data.id}@{common_vless_data.address}:{common_vless_data.port}?" \
               f"encryption={common_vless_data.encryption}&" \
               f"flow={common_vless_data.flow}&" \
               f"type={common_vless_data.network_type}&" \
               f"headerType={header_type}&" \
               f"security={common_vless_data.security}" \
               f"{self.security_fields(common_vless_data)}"

    def vless_uri_grpc_network(self):
        common_vless_data = self.CommonVlessData(config=self.config)
        service_name = common_vless_data.stream_settings.get("grpcSettings", {}).get("serviceName", "")
        return f"vless://{common_vless_data.id}@{common_vless_data.address}:{common_vless_data.port}?" \
               f"mode=gun&" \
               f"security={common_vless_data.security}&" \
               f"encryption={common_vless_data.encryption}&" \
               f"flow={common_vless_data.flow}&" \
               f"type={common_vless_data.network_type}&" \
               f"serviceName={service_name}" \
               f"{self.security_fields(common_vless_data)}"

    def vless_uri_xhttp_network(self):
        common_vless_data = self.CommonVlessData(config=self.config)
        path = quote(common_vless_data.stream_settings.get("xhttpSettings", {}).get("path", ""), safe="")
        vless_host = common_vless_data.stream_settings.get("xhttpSettings", {}).get("host", "")
        vless_host = vless_host.replace(" ", "").strip()
        return f"vless://{common_vless_data.id}@{common_vless_data.address}:{common_vless_data.port}?" \
               f"mode=auto&" \
               f"host={vless_host}&" \
               f"path={path}&" \
               f"security={common_vless_data.security}&" \
               f"encryption={common_vless_data.encryption}&" \
               f"flow={common_vless_data.flow}&" \
               f"type={common_vless_data.network_type}" \
               f"{self.security_fields(common_vless_data)}"

    def vless_uri_httpupgrade_network(self):
        common_vless_data = self.CommonVlessData(config=self.config)
        path = quote(common_vless_data.stream_settings.get("httpupgradeSettings", {}).get("path", ""), safe="")
        vless_host = common_vless_data.stream_settings.get("httpupgradeSettings", {}).get("host", "")
        vless_host = vless_host.replace(" ", "").strip()
        return f"vless://{common_vless_data.id}@{common_vless_data.address}:{common_vless_data.port}?" \
               f"mode=auto&" \
               f"host={vless_host}&" \
               f"path={path}&" \
               f"security={common_vless_data.security}&" \
               f"encryption={common_vless_data.encryption}&" \
               f"flow={common_vless_data.flow}&" \
               f"type={common_vless_data.network_type}" \
               f"{self.security_fields(common_vless_data)}"


class ShadowSocksProtocol(VlessProtocol):
    PROTOCOL = "ss"

    def _get_data_from_uri(self) -> Optional[XrayDecodeData]:
        if "@" in self.uri:
            return self.get_vless_data()

        decode_data = Base64Helper.decode_base64(self.uri_body)
        if not decode_data:
            return
        part_result = urlparse(f"ss://{decode_data}")
        return self.get_vless_data(pars_result=part_result)

    def _get_uri_from_config(self) -> str:
        base_config = self.config.get("outbounds", [{}])[0].get("settings", {}).get("servers", [{}])[0]
        address = base_config.get("address")
        port = base_config.get("port")
        method = base_config.get("method", "")
        password = base_config.get("password", "")
        body_base = f"{method}:{password}"
        encode_data = Base64Helper.encode_base64(body_base)
        return f"ss://{encode_data}@{address}:{port}"


class TrojanProtocol(VlessProtocol):
    PROTOCOL = "trojan"

    def _get_uri_from_config(self) -> str:  # pragma: no cover
        return ""


class SocksProtocol(VlessProtocol):
    PROTOCOL = "socks"

    def _get_uri_from_config(self) -> str:  # pragma: no cover
        return ""


class ProtocolHelper:
    PROTOCOL_CLASS = {
        "vmess": VmessProtocol,
        "vless": VlessProtocol,
        "ss": ShadowSocksProtocol,
        "shadowsocks": ShadowSocksProtocol,
        "trojan": TrojanProtocol,
        "socks": SocksProtocol
    }

    def __init__(self, uri: str = None, config: dict = None):
        self.uri = uri
        self.config = config

    def get_protocol_object(self, protocol) -> AbstractXrayProtocol:
        protocol_class = self.PROTOCOL_CLASS.get(protocol)
        if protocol_class is None:
            raise ValueError("not found protocol")
        return protocol_class(uri=self.uri, config=self.config)

    def get_data(self) -> Optional[XrayDecodeData]:
        if self.uri:
            self.uri = AbstractXrayProtocol.clean_uri_string(uri=self.uri)
            protocol = urlparse(self.uri).scheme
        elif self.config:
            protocol = self.config.get("outbounds", [{}])[0].get("protocol", "")
        else:
            return None
        protocol_obj = self.get_protocol_object(protocol=protocol)
        return protocol_obj.get_data()


# if __name__ == '__main__':
#     import json
#
#     with open("../temp.json", "r") as file:
#         json_data = json.load(file)
#     xray = ProtocolHelper(config=json_data).get_data()
#     print(xray.uri)



import logging
from typing import Dict, Union

from .base_xray_config import BASE_XRAY_CONFIG
from .config_creator import ConfigGenerator
from .config_type import EConfigType
from .consts import *
from .server_config import ServerConfig
from .utils import Utils
from .xray_config import XrayConfig, DEFAULT_NETWORK, HTTP


class XrayConfigUtil:
    class Result:
        def __init__(self, status: bool = False, config_dict: dict = None, error_message: str = None):
            self.status = status
            self.config_dict = config_dict
            self.error_message = error_message

    def get_xray_config(self, uri, port: int = None) -> Result:
        try:
            config_gen = ConfigGenerator(uri)
            created = config_gen.create_config_object()
            if not created:
                return self.Result(error_message=f"uri incorrect uri={uri}")
            config: "ServerConfig" = config_gen.config
            if not config:
                return self.Result(error_message=f"input config is not valid uri={uri}")
            if config.configType == EConfigType.CUSTOM.value[1]:
                config_dict = config.configType
            else:
                outbound: "XrayConfig.OutboundBean" = config.get_proxy_outbound()
                if not outbound:
                    return self.Result(error_message=f"input config is not valid uri={uri}")
                config_dict = self.get_xray_non_custom_config(outbound)
            try:
                config_dict["inbounds"][0]["port"] = port
                config_dict["inbounds"][1]["port"] = port + 1
            except Exception as e:
                return self.Result(error_message=f"{port} set_inbound_port {e}")
            return self.Result(status=True, config_dict=config_dict)

        except Exception as e:
            message = f"error in get v2ray config. e: {e}"
            logging.error(message)
            logging.exception(e)
            return self.Result(error_message=message)

    def get_xray_non_custom_config(self, outbound: "XrayConfig.OutboundBean") -> dict:
        v2ray_config = BASE_XRAY_CONFIG.copy()
        v2ray_config["log"]["loglevel"] = PREF_LOGLEVEL
        v2ray_config = self.inbounds(v2ray_config) or v2ray_config
        outbound = self.http_request_object(outbound) or outbound
        v2ray_config["outbounds"][0] = outbound.dict()
        v2ray_config = self.routing(v2ray_config) or v2ray_config
        # self.fakedns(v2ray_config)
        v2ray_config = self.dns(v2ray_config) or v2ray_config
        if PREF_LOCAL_DNS_ENABLED:
            v2ray_config = self.custom_local_dns(v2ray_config)
        if not PREF_SPEED_ENABLED:
            v2ray_config["stats"] = None
            v2ray_config["policy"] = None
        return v2ray_config

    @staticmethod
    def inbounds(v2ray_config) -> Union[bool, Dict]:
        try:
            for cur_inbound in v2ray_config["inbounds"]:
                if PREF_PROXY_SHARING:
                    cur_inbound["listen"] = "127.0.0.1"
            v2ray_config["inbounds"][0]["port"] = PORT_SOCKS
            fakedns = False
            sniff_all_tls_and_http = True
            v2ray_config["inbounds"][0]["sniffing"]["enabled"] = fakedns or sniff_all_tls_and_http
            if not sniff_all_tls_and_http:
                v2ray_config["inbounds"][0]["sniffing"]["destOverride"].clear()
            if fakedns:
                v2ray_config["inbounds"][0]["sniffing"]["destOverride"].append("fakedns")

            v2ray_config["inbounds"][1]["port"] = PORT_HTTP
        except Exception as e:
            # logging.exception(e)
            return False
        return v2ray_config

    """
    def fakedns(self, v2ray_config):
        if settings_storage.decode_bool(AppConfig.PREF_FAKE_DNS_ENABLED, False):
            v2ray_config["fakedns"] = [{}]
            for outbound in v2ray_config["outbounds"]:
                if outbound["protocol"] == "freedom":
                    outbound["settings"]["domainStrategy"] = "UseIP"
    """

    @staticmethod
    def routing(v2ray_config) -> Union[bool, Dict]:
        try:
            """
            routing_user_rule(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_AGENT, ""),
                              AppConfig.TAG_AGENT, v2ray_config)
            routing_user_rule(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_DIRECT, ""),
                              AppConfig.TAG_DIRECT, v2ray_config)
            routing_user_rule(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_BLOCKED, ""),
                              AppConfig.TAG_BLOCKED, v2ray_config)
            """

            v2ray_config["routing"]["domainStrategy"] = "IPIfNonMatch"

            """
            v2ray_config["routing"]["domainMatcher"] = "mph"  # Uncomment if needed

            routing_mode = 0

            googleapis_route = {
                "type": "field",
                "outboundTag": TAG_AGENT,
                "domain": ["domain:googleapis.cn"]
            }

            if routing_mode == ERoutingMode.BYPASS_LAN.value:
                routing_geo("ip", "private", AppConfig.TAG_DIRECT, v2ray_config)
            elif routing_mode == ERoutingMode.BYPASS_MAINLAND.value:
                routing_geo("", "cn", AppConfig.TAG_DIRECT, v2ray_config)
                v2ray_config["routing"]["rules"].insert(0, googleapis_route)
            elif routing_mode == ERoutingMode.BYPASS_LAN_MAINLAND.value:
                routing_geo("ip", "private", AppConfig.TAG_DIRECT, v2ray_config)
                routing_geo("", "cn", AppConfig.TAG_DIRECT, v2ray_config)
                v2ray_config["routing"]["rules"].insert(0, googleapis_route)
            elif routing_mode == ERoutingMode.GLOBAL_DIRECT.value:
                global_direct = {
                    "type": "field",
                    "outboundTag": AppConfig.TAG_DIRECT,
                    "port": "0-65535"
                }
                v2ray_config["routing"]["rules"].insert(0, global_direct)
            """

        except Exception as e:
            # logging.exception(e)
            return False
        return v2ray_config

    """
    def routing_geo(self, ip_or_domain, code, tag, v2ray_config):
        try:
            if code:
                # IP
                if ip_or_domain == "ip" or not ip_or_domain:
                    rules_ip = {
                        "type": "field",
                        "outboundTag": tag,
                        "ip": ["geoip:" + code]
                    }
                    v2ray_config["routing"]["rules"].append(rules_ip)

                if ip_or_domain == "domain" or not ip_or_domain:
                    # Domain
                    rules_domain = {
                        "type": "field",
                        "outboundTag": tag,
                        "domain": ["geosite:" + code]
                    }
                    v2ray_config["routing"]["rules"].append(rules_domain)
        except Exception as e:
            print(e)

    def routing_user_rule(self, user_rule, tag, v2ray_config):
        try:
            if user_rule:
                rules_domain = {
                    "type": "field",
                    "outboundTag": tag,
                    "domain": []
                }

                rules_ip = {
                    "type": "field",
                    "outboundTag": tag,
                    "ip": []
                }

                for item in map(str.strip, user_rule.split(",")):
                    if Utils.is_ip_address(item) or item.startswith("geoip:"):
                        rules_ip["ip"].append(item)
                    elif item:
                        rules_domain["domain"].append(item)

                if rules_domain["domain"]:
                    v2ray_config["routing"]["rules"].append(rules_domain)
                if rules_ip["ip"]:
                    v2ray_config["routing"]["rules"].append(rules_ip)
        except Exception as e:
            print(e)

    def user_rule_to_domain(self, user_rule):
        domain = []
        for item in map(str.strip, user_rule.split(",")):
            if item.startswith("geosite:") or item.startswith("domain:"):
                domain.append(item)
        return domain

    """

    @staticmethod
    def custom_local_dns(v2ray_config) -> Union[bool, Dict]:
        try:
            """
            if settings_storage.decode_bool(AppConfig.PREF_FAKE_DNS_ENABLED):
                geosite_cn = ["geosite:cn"]
                proxy_domain = user_rule_to_domain(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_AGENT))
                direct_domain = user_rule_to_domain(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_DIRECT))
                v2ray_config["dns"]["servers"].insert(0, {
                    "address": "fakedns",
                    "domains": geosite_cn + proxy_domain + direct_domain
                })
            """
            remote_dns = Utils.get_remote_dns_servers()
            if not any(
                    e for e in v2ray_config["inbounds"] if e["protocol"] == "dokodemo-door" and e["tag"] == "dns-in"):
                dns_inbound_settings = {
                    "address": remote_dns[0] if Utils.is_pure_ip_address(remote_dns[0]) else "1.1.1.1",
                    "port": 53,
                    "network": "tcp,udp"
                }
                local_dns_port = int(PORT_LOCAL_DNS)
                v2ray_config["inbounds"].append({
                    "tag": "dns-in",
                    "port": local_dns_port,
                    "listen": "127.0.0.1",
                    "protocol": "dokodemo-door",
                    "settings": dns_inbound_settings,
                    "sniffing": None
                })

            if not any(e for e in v2ray_config["outbounds"] if e["protocol"] == "dns" and e["tag"] == "dns-out"):
                v2ray_config["outbounds"].append({
                    "protocol": "dns",
                    "tag": "dns-out",
                    "settings": None,
                    "streamSettings": None,
                    "mux": None
                })

            v2ray_config["routing"]["rules"].insert(0, {
                "type": "field",
                "inboundTag": ["dns-in"],
                "outboundTag": "dns-out",
                "domain": None
            })
        except Exception as e:
            # logging.exception(e)
            return False
        return v2ray_config

    @staticmethod
    def dns(v2ray_config) -> Union[bool, Dict]:
        try:
            hosts = {}
            servers = []
            remote_dns = Utils.get_remote_dns_servers()
            servers.extend(remote_dns)

            """
            proxy_domain = user_rule_to_domain(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_AGENT))

            servers.extend(remote_dns)
            if proxy_domain:
                servers.append({
                    "address": remote_dns[0],
                    "port": 53,
                    "domains": proxy_domain
                })

            direct_domain = user_rule_to_domain(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_DIRECT))
            routing_mode = 0
            if direct_domain or routing_mode == ERoutingMode.BYPASS_MAINLAND.value or routing_mode == 
                ERoutingMode.BYPASS_LAN_MAINLAND.value:
                domestic_dns = Utils.get_domestic_dns_servers()
                geosite_cn = ["geosite:cn"]
                geoip_cn = ["geoip:cn"]
                if direct_domain:
                    servers.append({
                        "address": domestic_dns[0],
                        "port": 53,
                        "domains": direct_domain,
                        "ip": geoip_cn
                    })
                if routing_mode == ERoutingMode.BYPASS_MAINLAND.value or routing_mode == 
                    ERoutingMode.BYPASS_LAN_MAINLAND.value:
                    servers.append({
                        "address": domestic_dns[0],
                        "port": 53,
                        "domains": geosite_cn,
                        "ip": geoip_cn
                    })
                if Utils.is_pure_ip_address(domestic_dns[0]):
                    v2ray_config["routing"]["rules"].insert(0, {
                        "type": "field",
                        "outboundTag": AppConfig.TAG_DIRECT,
                        "port": "53",
                        "ip": [domestic_dns[0]],
                        "domain": None
                    })

            blk_domain = user_rule_to_domain(settings_storage.decode_string(AppConfig.PREF_V2RAY_ROUTING_BLOCKED))
            if blk_domain:
                hosts.update({item: "127.0.0.1" for item in blk_domain})
            """

            hosts["domain:googleapis.cn"] = "googleapis.com"

            v2ray_config["dns"] = {
                "servers": servers,
                "hosts": hosts
            }

            if Utils.is_pure_ip_address(remote_dns[0]):
                v2ray_config["routing"]["rules"].insert(0, {
                    "type": "field",
                    "outboundTag": TAG_AGENT,
                    "port": "53",
                    "ip": [remote_dns[0]],
                    "domain": None
                })
        except Exception as e:
            # logging.exception(e)
            return False
        return v2ray_config

    @staticmethod
    def http_request_object(outbound: XrayConfig.OutboundBean):
        try:
            if (outbound.streamSettings and
                    outbound.streamSettings.network == DEFAULT_NETWORK and
                    outbound.streamSettings.tcpSettings.header.type == HTTP):
                path = outbound.streamSettings.tcpSettings.header.request.path
                host = outbound.streamSettings.tcpSettings.header.request.headers.Host

                request_dict = {
                    "version": "1.1",
                    "method": "GET",
                    "headers": {
                        "User-Agent": [
                            "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/53.0.2785.143 Safari/537.36",
                            "Mozilla/5.0 (iPhone; CPU iPhone OS 10_0_2 like Mac OS X) AppleWebKit/601.1 (KHTML, like Gecko) CriOS/53.0.2785.109 Mobile/14A456 Safari/601.1.46"],
                        "Accept-Encoding": ["gzip, deflate"],
                        "Connection": ["keep-alive"],
                        "Pragma": "no-cache"
                    }
                }

                outbound.streamSettings.tcpSettings.header.request = request_dict
                outbound.streamSettings.tcpSettings.header.request['path'] = ["/"] if not path else path
                outbound.streamSettings.tcpSettings.header.request['headers']['Host'] = host

        except Exception as e:
            # logging.exception(e)
            return False
        return outbound



import json
import logging
import re
import urllib.parse
from typing import Optional

from .consts import HTTP_PROTOCOL, HTTPS_PROTOCOL
from .config_type import EConfigType
from .server_config import ServerConfig
from .utils import Utils
from .xray_config import DEFAULT_SECURITY, XrayConfig, TLS
from urllib.parse import unquote, urlparse, parse_qs


class ConfigGenerator:

    def __init__(self, uri):
        self.uri = uri
        self.config = {}

    def create_config_object(self) -> bool:
        try:
            if self.uri is None or self.uri.strip() == "":
                # logging.info("uri is empty.")
                return False

            if self.uri.startswith(HTTP_PROTOCOL) or self.uri.startswith(HTTPS_PROTOCOL):
                # logging.info("uri is http or https protocol.")
                return False

            config: Optional[ServerConfig] = None
            allow_insecure = False

            if self.uri.startswith(EConfigType.VMESS.value[1]):
                config = ServerConfig.create(EConfigType.VMESS)
                if config.outboundBean:
                    stream_setting: XrayConfig.OutboundBean.StreamSettingsBean = config.outboundBean.streamSettings
                else:
                    # logging.info("stream setting nothing")
                    return False

                if not self.try_parse_new_vmess(config, allow_insecure):
                    if "?" in self.uri:
                        if not self.try_resolve_vmess_4_kitsunebi(config):
                            # logging.info("incorrect protocol")
                            return False
                    else:
                        result = self.uri.replace(EConfigType.VMESS.value[1], "")
                        result = Utils.decode(result)
                        if result == "":
                            # logging.info("decoding failed.")
                            return False

                        vmess_qr_code = json.loads(result)
                        if (not vmess_qr_code.get("add") or
                                not vmess_qr_code.get("port") or
                                not vmess_qr_code.get("id") or
                                not vmess_qr_code.get("net")):
                            # logging.info("incorrect protocol.")
                            return False

                        config.remarks = vmess_qr_code.get("ps")
                        vnext = config.outboundBean.settings.vnext[0]
                        vnext.address = vmess_qr_code.get("add")
                        vnext.port = int(vmess_qr_code.get("port"))
                        vnext.users[0].id = vmess_qr_code.get("id")
                        vnext.users[0].security = DEFAULT_SECURITY if not vmess_qr_code.get(
                            "scy") else vmess_qr_code.get("scy")
                        vnext.users[0].alterId = 0 if not vmess_qr_code.get("aid") else int(vmess_qr_code.get("aid"))
                        sni = stream_setting.populate_transport_settings(
                            vmess_qr_code.get("net"),
                            vmess_qr_code.get("type"),
                            vmess_qr_code.get("host"),
                            vmess_qr_code.get("path"),
                            vmess_qr_code.get("path"),
                            vmess_qr_code.get("host"),
                            vmess_qr_code.get("path"),
                            vmess_qr_code.get("type"),
                            vmess_qr_code.get("path")
                        )

                        fingerprint = vmess_qr_code.get("fp") if vmess_qr_code.get("fp") is not None else (
                            stream_setting.tlsSettings.fingerprint) if stream_setting.tlsSettings is not None else None

                        stream_setting.populate_tls_settings(
                            vmess_qr_code.get("tls"), allow_insecure,
                            sni if vmess_qr_code.get("sni") is None else vmess_qr_code.get("sni"),
                            fingerprint, vmess_qr_code.get("alpn"), None, None, None
                        )

            elif self.uri.startswith(EConfigType.SHADOWSOCKS.value[1]):
                config: ServerConfig = ServerConfig.create(EConfigType.SHADOWSOCKS)
                if not self.try_resolve_sip002(config):
                    result = self.uri.replace(EConfigType.SHADOWSOCKS.value[1], "")
                    index_split = result.find("#")
                    if index_split > 0:
                        try:
                            config.remarks = Utils.url_decode(result[index_split + 1:])
                        except Exception as e:
                            logging.error(f"uri: '{self.uri}' e: '{e}'")
                            # logging.exception(e)
                        result = result[:index_split]

                    index_s = result.find("@")
                    if index_s > 0:
                        result = Utils.decode(result[:index_s]) + result[index_s:]
                    else:
                        result = Utils.decode(result)

                    legacy_pattern = r"^(.+?):(.*)@(.+?):(\d+?)/?$"
                    match = re.match(legacy_pattern, result)
                    if not match:
                        # logging.info("incorrect protocol.")
                        return False

                    server = config.outboundBean.settings.servers[0]
                    server.address = match.group(3).strip("[]")
                    server.port = int(match.group(4))
                    server.password = match.group(2)
                    server.method = match.group(1).lower()

            elif self.uri.startswith(EConfigType.SOCKS.value[1]):
                result = self.uri.replace(EConfigType.SOCKS.value[1], "")
                index_split = result.find("#")
                config: ServerConfig = ServerConfig.create(EConfigType.SOCKS)
                if index_split > 0:
                    try:
                        config.remarks = Utils.url_decode(result[index_split + 1:])
                    except Exception as e:
                        logging.error(f"uri: '{self.uri}' e: '{e}'")
                        # logging.exception(e)
                    result = result[:index_split]

                index_s = result.find("@")
                if index_s > 0:
                    result = Utils.decode(result[:index_s]) + result[index_s:]
                else:
                    result = Utils.decode(result)

                legacy_pattern = r"^(.*):(.*)@(.+?):(\d+?)$"
                match = re.match(legacy_pattern, result)
                if not match:
                    # logging.info("incorrect protocol.")
                    return False

                server = config.outboundBean.settings.servers[0]
                server.address = match.group(3).strip("[]")
                server.port = int(match.group(4))
                socks_users_bean = XrayConfig.OutboundBean.OutSettingsBean.ServersBean.SocksUsersBean()
                socks_users_bean.user = match.group(1).lower()
                socks_users_bean.passw = match.group(2)
                server.users = [socks_users_bean]

            elif self.uri.startswith(EConfigType.TROJAN.value[1]):
                uri = urllib.parse.urlparse(urllib.parse.unquote(self.uri))
                config: ServerConfig = ServerConfig.create(EConfigType.TROJAN)
                config.remarks = Utils.url_decode(uri.fragment or "")

                flow = ""
                if uri.query:
                    query_params = parse_qs(uri.query)
                    query_param = {k: unquote(v[0]) for k, v in query_params.items()}
                    sni = config.outboundBean.streamSettings.populate_transport_settings(
                        query_param.get("type", "tcp"),
                        query_param.get("headerType"),
                        query_param.get("host"),
                        query_param.get("path"),
                        query_param.get("seed"),
                        query_param.get("quicSecurity"),
                        query_param.get("key"),
                        query_param.get("mode"),
                        query_param.get("serviceName")
                    )
                    fingerprint = query_param.get("fp", "")
                    config.outboundBean.streamSettings.populate_tls_settings(
                        query_param.get("security", TLS),
                        allow_insecure,
                        query_param.get("sni", sni),
                        fingerprint,
                        query_param.get("alpn"),
                        None,
                        None,
                        None
                    )
                    flow = query_param.get("flow", "")
                else:
                    fingerprint = config.outboundBean.streamSettings.tlsSettings.fingerprint
                    config.outboundBean.streamSettings.populate_tls_settings(
                        TLS,
                        allow_insecure,
                        "",
                        fingerprint,
                        None,
                        None,
                        None,
                        None
                    )

                server = config.outboundBean.settings.servers[0]
                server.address = uri.hostname
                server.port = uri.port
                server.password = uri.username
                server.flow = flow

            elif self.uri.startswith(EConfigType.VLESS.value[1]):
                uri = urllib.parse.urlparse(urllib.parse.unquote(self.uri))
                query_params = parse_qs(uri.query)
                query_param = {k: unquote(v[0]) for k, v in query_params.items()}

                # for it in uri.query.split("&"):
                #     if "?" in it:
                #         it = it.split("?")[0]
                #     try:
                #         k, v = it.split("=", 1)
                #         query_param[k] = v
                #     except Exception as e:
                #         logging.error(f"uri: '{it}' e: '{e}'")
                #         return False

                config: ServerConfig = ServerConfig.create(EConfigType.VLESS)
                if config.outboundBean:
                    stream_setting = config.outboundBean.streamSettings
                else:
                    return False
                # fingerprint = stream_setting.tlsSettings.fingerprint

                config.remarks = Utils.url_decode(uri.fragment or "")
                vnext = config.outboundBean.settings.vnext[0]
                vnext.address = uri.hostname
                vnext.port = uri.port
                vnext.users[0].id = uri.username
                vnext.users[0].encryption = query_param.get("encryption", "none")
                vnext.users[0].flow = query_param.get("flow", "")

                sni = stream_setting.populate_transport_settings(
                    query_param.get("type", "tcp"),
                    query_param.get("headerType"),
                    query_param.get("host"),
                    query_param.get("path"),
                    query_param.get("seed"),
                    query_param.get("quicSecurity"),
                    query_param.get("key"),
                    query_param.get("mode"),
                    query_param.get("serviceName")
                )
                fingerprint = query_param.get("fp", "")
                pbk = query_param.get("pbk", "")
                sid = query_param.get("sid", "")
                spx = Utils.url_decode(query_param.get("spx", ""))
                stream_setting.populate_tls_settings(
                    query_param.get("security", ""),
                    allow_insecure,
                    query_param.get("sni", sni),
                    fingerprint,
                    query_param.get("alpn"),
                    pbk,
                    sid,
                    spx
                )

            if config is None:
                return False
        except Exception as e:
            logging.exception(f"error in create config object. e: {e}.")
            # logging.error(e)
            return False

        self.config = config
        return True

    def try_parse_new_vmess(self, config: ServerConfig, allow_insecure: bool):
        try:
            uri = urllib.parse.urlparse(urllib.parse.unquote(self.uri))
            if uri.scheme != "vmess":
                return False
            match = re.match(
                r"(tcp|http|ws|httpupgrade|xhttp|kcp|quic|grpc)(\+tls)?:([0-9a-z]{8}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{4}-[0-9a-z]{12})",
                str(uri.username))
            if not match:
                return False

            protocol, tls_str, uuid, alter_id = match.groups()
            tls = tls_str.strip() != ""

            query_params = parse_qs(uri.query)
            query_param = {k: unquote(v[0]) for k, v in query_params.items()}

            stream_setting = config.outboundBean.streamSettings
            config.remarks = Utils.url_decode(uri.fragment or "")
            vnext = config.outboundBean.settings.vnext[0]
            vnext.address = uri.hostname
            vnext.port = uri.port
            vnext.users[0].id = uuid
            vnext.users[0].security = DEFAULT_SECURITY
            vnext.users[0].alterId = int(alter_id)

            fingerprint = stream_setting.tlsSettings.fingerprint
            sni = stream_setting.populate_transport_settings(
                protocol,
                query_param.get("type"),
                query_param.get("host", "").split("|")[0],
                query_param.get("path", "") if query_param.get("path", "").strip() != "/" else "",
                query_param.get("seed"),
                query_param.get("security"),
                query_param.get("key"),
                query_param.get("mode"),
                query_param.get("serviceName")
            )
            stream_setting.populate_tls_settings(
                TLS if tls else "", allow_insecure, sni, fingerprint, None, None, None, None
            )
            return True

        except Exception as e:
            logging.error(f"uri: '{self.uri}' e: '{e}'")
            # logging.exception(e)
            return False

    def try_resolve_vmess_4_kitsunebi(self, config):
        try:
            result = self.uri.replace(EConfigType.VMESS.value[1], "")
            index_split = result.find("?")
            if index_split > 0:
                result = result[:index_split]
            result = Utils.decode(result)

            arr1 = result.split('@')
            if len(arr1) != 2:
                return False
            arr21 = arr1[0].split(':')
            arr22 = arr1[1].split(':')
            if len(arr21) != 2:
                return False

            config.remarks = "Alien"
            if config.outboundBean and config.outboundBean.settings and config.outboundBean.settings.vnext:
                vnext = config.outboundBean.settings.vnext[0]
                vnext.address = arr22[0]
                vnext.port = int(arr22[1])
                vnext.users[0].id = arr21[1]
                vnext.users[0].security = arr21[0]
                vnext.users[0].alterId = 0
            return True

        except Exception as e:
            logging.error(f"uri: '{self.uri}' e: '{e}'")
            # logging.exception(e)
            return False

    def try_resolve_sip002(self, config):
        try:
            uri = urllib.parse.urlparse(urllib.parse.unquote(self.uri))
            config.remarks = Utils.url_decode(uri.fragment or "")

            if uri.username and ":" in uri.username:
                arr_user_info = uri.username.split(":")
                if len(arr_user_info) != 2:
                    return False
                method = arr_user_info[0].strip()
                password = Utils.url_decode(arr_user_info[1].strip())
            else:
                base64_decode = Utils.decode(uri.username)
                arr_user_info = base64_decode.split(":")
                if len(arr_user_info) < 2:
                    return False
                method = arr_user_info[0].strip()
                password = base64_decode.split(":", 1)[1]

            if config.outboundBean and config.outboundBean.settings and config.outboundBean.settings.servers:
                server = config.outboundBean.settings.servers[0]
                server.address = uri.hostname
                server.port = uri.port
                server.password = password
                server.method = method

            return True
        except Exception as e:
            logging.error(f"uri: '{self.uri}' e: '{e}'")
            # logging.exception(e)
            return False



import json

from .config_type import EConfigType

DEFAULT_PORT = 443
DEFAULT_SECURITY = "auto"
DEFAULT_LEVEL = 8
DEFAULT_NETWORK = "tcp"

TLS = "tls"
REALITY = "reality"
HTTP = "http"


class BaseConfig:
    @classmethod
    def get_value(cls, v):
        if type(v) in (list, tuple):
            return cls.convert_list_to_dict(v)
        elif type(v) not in (float, int, str, dict, bool):
            return v.dict()
        return v

    @classmethod
    def convert_list_to_dict(cls, input_list: list):
        """
            converting list to dict

        Args:
            input_list (list): input list

        Returns:
            dict: dict of converted list
        """
        return [cls.get_value(input_data) for input_data in input_list]

    def dict(self):
        """
        change a list to a dictionary.

        Returns:
            (dict): dictionary from input list.
        """
        data = self.__dict__
        new_data = {}
        for k, v in data.items():
            if not v and not v == 0:
                continue
            new_data[k] = self.get_value(v)
        return new_data


class XrayConfig(BaseConfig):

    def __init__(self, stats=None, log=None, policy=None, inbounds=None, outbounds=None,
                 dns=None, routing=None, api=None, transport=None, reverse=None, fakedns=None,
                 browser_forwarder=None):
        self.stats = stats
        self.log = log
        self.policy = policy
        self.inbounds = inbounds or []
        self.outbounds = outbounds or []
        self.dns = dns
        self.routing = routing
        self.api = api
        self.transport = transport
        self.reverse = reverse
        self.fakedns = fakedns
        self.browserForwarder = browser_forwarder

    class LogBean(BaseConfig):
        def __init__(self, access, error, loglevel=None, dns_log=None):
            self.access = access
            self.error = error
            self.loglevel = loglevel
            self.dnsLog = dns_log

    class InboundBean(BaseConfig):
        def __init__(self, tag, port, protocol, listen=None, settings=None, sniffing=None,
                     stream_settings=None, allocate=None):
            self.tag = tag
            self.port = port
            self.protocol = protocol
            self.listen = listen
            self.settings = settings
            self.sniffing = sniffing
            self.streamSettings = stream_settings
            self.allocate = allocate

        class InSettingsBean(BaseConfig):
            def __init__(self, auth=None, udp=None, user_level=None, address=None, port=None, network=None):
                self.auth = auth
                self.udp = udp
                self.userLevel = user_level
                self.address = address
                self.port = port
                self.network = network

        class SniffingBean(BaseConfig):
            def __init__(self, enabled, dest_override, metadata_only=None):
                self.enabled = enabled
                self.destOverride = dest_override
                self.metadataOnly = metadata_only

    class OutboundBean(BaseConfig):
        def __init__(self, tag="proxy", protocol=None, settings=None, stream_settings=None,
                     proxy_settings=None, send_through=None, mux=None):
            self.tag = tag
            self.protocol = protocol
            self.settings = settings
            self.streamSettings = stream_settings
            self.proxySettings = proxy_settings
            self.sendThrough = send_through
            self.mux = mux or self.MuxBean(False)

        class OutSettingsBean(BaseConfig):
            def __init__(self, vnext=None, servers=None, response=None, network=None,
                         address=None, port=None, domain_strategy=None, redirect=None,
                         user_level=None, inbound_tag=None, secret_key=None, peers=None):
                self.vnext = vnext or []
                self.servers = servers or []
                self.response = response
                self.network = network
                self.address = address
                self.port = port
                self.domainStrategy = domain_strategy
                self.redirect = redirect
                self.userLevel = user_level
                self.inboundTag = inbound_tag
                self.secretKey = secret_key
                self.peers = peers or []

            class VnextBean(BaseConfig):
                def __init__(self, address="", port=DEFAULT_PORT, users=None):
                    self.address = address
                    self.port = port
                    self.users = users or []

                class UsersBean(BaseConfig):
                    def __init__(self, id="", alter_id=None, security=DEFAULT_SECURITY,
                                 level=DEFAULT_LEVEL, encryption="", flow=""):
                        self.id = id
                        self.alterId = alter_id
                        self.security = security
                        self.level = level
                        self.encryption = encryption
                        self.flow = flow

            class ServersBean(BaseConfig):
                def __init__(self, address="", method="chacha20-poly1305", ota=False,
                             password="", port=DEFAULT_PORT, level=DEFAULT_LEVEL,
                             email=None, flow=None, iv_check=None, users=None):
                    self.address = address
                    self.method = method
                    self.ota = ota
                    self.password = password
                    self.port = port
                    self.level = level
                    self.email = email
                    self.flow = flow
                    self.ivCheck = iv_check
                    self.users = users or []

                class SocksUsersBean(BaseConfig):
                    def __init__(self, user="", passw="", level=DEFAULT_LEVEL):
                        self.user = user
                        self.passw = passw
                        self.level = level

            class Response(BaseConfig):
                def __init__(self, type):
                    self.type = type

            class WireGuardBean(BaseConfig):
                def __init__(self, public_key="", endpoint=""):
                    self.publicKey = public_key
                    self.endpoint = endpoint

        class StreamSettingsBean(BaseConfig):
            def __init__(self, network=DEFAULT_NETWORK, security="", tcp_settings=None,
                         kcp_settings=None, ws_settings=None,
                         httpupgrade_settings=None, xhttp_settings=None,
                         http_settings=None,
                         tls_settings=None, quic_settings=None, reality_settings=None,
                         grpc_settings=None, ds_settings=None, sockopt=None):
                self.network = network
                self.security = security
                self.tcpSettings = tcp_settings
                self.kcpSettings = kcp_settings
                self.wsSettings = ws_settings
                self.httpupgradeSettings = httpupgrade_settings
                self.xhttpSettings = xhttp_settings
                self.httpSettings = http_settings
                self.tlsSettings = tls_settings
                self.quicSettings = quic_settings
                self.realitySettings = reality_settings
                self.grpcSettings = grpc_settings
                self.dsSettings = ds_settings
                self.sockopt = sockopt

            class TcpSettingsBean(BaseConfig):
                def __init__(self, header=None, accept_proxy_protocol=None):
                    self.header = header or self.HeaderBean()
                    self.acceptProxyProtocol = accept_proxy_protocol

                class HeaderBean(BaseConfig):
                    def __init__(self, type="none", request=None, response=None):
                        self.type = type
                        self.request = request or self.RequestBean()
                        self.response = response

                    class RequestBean(BaseConfig):
                        def __init__(self, path=None, headers=None, version=None, method=None):
                            self.path = path or []
                            self.headers = headers or self.HeadersBean()
                            self.version = version
                            self.method = method

                        class HeadersBean(BaseConfig):
                            def __init__(self, host=None, user_agent=None, accept_encoding=None,
                                         connection=None, pragma=None):
                                self.Host = host or []
                                self.UserAgent = user_agent or []
                                self.AcceptEncoding = accept_encoding or []
                                self.Connection = connection or []
                                self.Pragma = pragma

            class KcpSettingsBean(BaseConfig):
                def __init__(self, mtu=1350, tti=50, uplink_capacity=12, downlink_capacity=100,
                             congestion=False, read_buffer_size=1, write_buffer_size=1,
                             header=None, seed=None):
                    self.mtu = mtu
                    self.tti = tti
                    self.uplinkCapacity = uplink_capacity
                    self.downlinkCapacity = downlink_capacity
                    self.congestion = congestion
                    self.readBufferSize = read_buffer_size
                    self.writeBufferSize = write_buffer_size
                    self.header = header or self.HeaderBean()
                    self.seed = seed

                class HeaderBean(BaseConfig):
                    def __init__(self, type="none"):
                        self.type = type

            class WsSettingsBean(BaseConfig):
                def __init__(self, path="", headers=None, max_early_data=None,
                             use_browser_forwarding=None, accept_proxy_protocol=None):
                    self.path = path
                    self.headers = headers or self.HeadersBean()
                    self.maxEarlyData = max_early_data
                    self.useBrowserForwarding = use_browser_forwarding
                    self.acceptProxyProtocol = accept_proxy_protocol

                class HeadersBean(BaseConfig):
                    def __init__(self, host=""):
                        self.Host = host

            class HttpupgradeSettingsBean(BaseConfig):
                def __init__(self, path="", host=None, accept_proxy_protocol=None):
                    self.path = path
                    self.host = host
                    self.acceptProxyProtocol = accept_proxy_protocol

            class XhttpSettingsBean(BaseConfig):
                def __init__(self, path="", host=None, mode=None, extra=None):
                    self.path = path
                    self.host = host
                    self.mode = mode
                    self.extra = extra

            class HttpSettingsBean(BaseConfig):
                def __init__(self, host=None, path=""):
                    self.host = host or []
                    self.path = path

            class TlsSettingsBean(BaseConfig):
                def __init__(self, allow_insecure=False, server_name="", alpn=None,
                             min_version=None, max_version=None, prefer_server_cipher_suites=None,
                             cipher_suites=None, fingerprint=None, certificates=None,
                             disable_system_root=None, enable_session_resumption=None,
                             show=False, public_key=None, short_id=None, spider_x=None):
                    self.allowInsecure = allow_insecure
                    self.serverName = server_name
                    self.alpn = alpn or []
                    self.minVersion = min_version
                    self.maxVersion = max_version
                    self.preferServerCipherSuites = prefer_server_cipher_suites
                    self.cipherSuites = cipher_suites
                    self.fingerprint = fingerprint
                    self.certificates = certificates or []
                    self.disableSystemRoot = disable_system_root
                    self.enableSessionResumption = enable_session_resumption
                    self.show = show
                    self.publicKey = public_key
                    self.shortId = short_id
                    self.spiderX = spider_x

            class QuicSettingBean(BaseConfig):
                def __init__(self, security="none", key="", header=None):
                    self.security = security
                    self.key = key
                    self.header = header or self.HeaderBean()

                class HeaderBean(BaseConfig):
                    def __init__(self, type="none"):
                        self.type = type

            class GrpcSettingsBean(BaseConfig):
                def __init__(self, service_name="", multi_mode=None):
                    self.serviceName = service_name
                    self.multiMode = multi_mode

            def populate_transport_settings(self, transport, header_type=None, host=None, path=None,
                                            seed=None, quic_security=None, key=None, mode=None,
                                            service_name=None):
                sni = ""
                self.network = transport
                if self.network == "tcp":
                    tcp_setting = self.TcpSettingsBean()
                    if header_type == HTTP:
                        tcp_setting.header.type = HTTP
                        if host or path:
                            request_obj = self.TcpSettingsBean.HeaderBean.RequestBean()
                            request_obj.headers.Host = [h.strip() for h in (host or "").split(",")
                                                        if h.strip()]
                            request_obj.path = [p.strip() for p in (path or "").split(",") if
                                                p.strip()]
                            tcp_setting.header.request = request_obj
                            sni = request_obj.headers.Host[0] if request_obj.headers.Host else sni
                    else:
                        tcp_setting.header.type = "none"
                        sni = host or ""
                    self.tcpSettings = tcp_setting
                elif self.network == "kcp":
                    kcp_setting = self.KcpSettingsBean()
                    kcp_setting.header.type = header_type or "none"
                    kcp_setting.seed = None if seed is None else seed
                    self.kcpSettings = kcp_setting
                elif self.network == "ws":
                    ws_setting = self.WsSettingsBean()
                    ws_setting.headers.Host = host or ""
                    sni = ws_setting.headers.Host
                    ws_setting.path = path or "/"
                    self.wsSettings = ws_setting
                elif self.network == "httpupgrade":
                    httpupgrade_setting = self.HttpupgradeSettingsBean()
                    httpupgrade_setting.host = host or ""
                    sni = host or ""
                    httpupgrade_setting.path = path or "/"
                    self.httpupgradeSettings = httpupgrade_setting
                elif self.network == "xhttp":
                    xhttp_setting = self.XhttpSettingsBean()
                    xhttp_setting.host = host or ""
                    sni = host or ""
                    xhttp_setting.path = path or "/"
                    self.xhttpSettings = xhttp_setting
                elif self.network in ("h2", "http"):
                    self.network = "h2"
                    h2_setting = self.HttpSettingsBean()
                    h2_setting.host = [h.strip() for h in (host or "").split(",") if h.strip()]
                    sni = h2_setting.host[0] if h2_setting.host else sni
                    h2_setting.path = path or "/"
                    self.httpSettings = h2_setting
                elif self.network == "quic":
                    quic_setting = self.QuicSettingBean()
                    quic_setting.security = quic_security or "none"
                    quic_setting.key = key or ""
                    quic_setting.header.type = header_type or "none"
                    self.quicSettings = quic_setting
                elif self.network == "grpc":
                    grpc_setting = self.GrpcSettingsBean()
                    grpc_setting.multiMode = mode == "multi"
                    grpc_setting.serviceName = service_name or ""
                    sni = host or ""
                    self.grpcSettings = grpc_setting
                return sni

            def populate_tls_settings(self, stream_security, allow_insecure, sni, fingerprint, alpns,
                                      public_key, short_id, spider_x):
                self.security = stream_security
                tls_setting = self.TlsSettingsBean(
                    allow_insecure=allow_insecure,
                    server_name=sni,
                    fingerprint=fingerprint,
                    alpn=[a.strip() for a in (alpns or "").split(",") if a.strip()],
                    public_key=public_key,
                    short_id=short_id,
                    spider_x=spider_x
                )
                if self.security == TLS:
                    self.tlsSettings = tls_setting
                    self.realitySettings = None
                elif self.security == REALITY:
                    self.tlsSettings = None
                    self.realitySettings = tls_setting

        class MuxBean(BaseConfig):
            def __init__(self, enabled, concurrency=8):
                self.enabled = enabled
                self.concurrency = concurrency

        def get_server_address(self):
            if (self.protocol.lower() == EConfigType.VMESS.name.lower() or
                    self.protocol.lower() == EConfigType.VLESS.name.lower()):
                return self.settings.vnext[0].address
            elif (self.protocol.lower() == EConfigType.SHADOWSOCKS.name.lower() or
                  self.protocol.lower() == EConfigType.SOCKS.name.lower() or
                  self.protocol.lower() == EConfigType.TROJAN.name.lower()):
                return self.settings.servers[0].address
            elif self.protocol.lower() == EConfigType.WIREGUARD.name.lower():
                return self.settings.peers[0].endpoint.split(":")[0]
            return None

        def get_server_port(self):
            if (self.protocol.lower() == EConfigType.VMESS.name.lower() or
                    self.protocol.lower() == EConfigType.VLESS.name.lower()):
                return self.settings.vnext[0].port
            elif (self.protocol.lower() == EConfigType.SHADOWSOCKS.name.lower() or
                  self.protocol.lower() == EConfigType.SOCKS.name.lower() or
                  self.protocol.lower() == EConfigType.TROJAN.name.lower()):
                return self.settings.servers[0].port
            elif self.protocol.lower() == EConfigType.WIREGUARD.name.lower():
                return int(self.settings.peers[0].endpoint.split(":")[-1])
            return None

        def get_password(self):
            if (self.protocol.lower() == EConfigType.VMESS.name.lower() or
                    self.protocol.lower() == EConfigType.VLESS.name.lower()):
                return self.settings.vnext[0].users[0].id
            elif (self.protocol.lower() == EConfigType.SHADOWSOCKS.name.lower() or
                  self.protocol.lower() == EConfigType.TROJAN.name.lower()):
                return self.settings.servers[0].password
            elif self.protocol.lower() == EConfigType.SOCKS.name.lower():
                return self.settings.servers[0].users[0].passw
            elif self.protocol.lower() == EConfigType.WIREGUARD.name.lower():
                return self.settings.secretKey
            return None

        def get_security_encryption(self):
            if self.protocol.lower() == EConfigType.VMESS.name.lower():
                return self.settings.vnext[0].users[0].security
            elif self.protocol.lower() == EConfigType.VLESS.name.lower():
                return self.settings.vnext[0].users[0].encryption
            elif self.protocol.lower() == EConfigType.SHADOWSOCKS.name.lower():
                return self.settings.servers[0].method
            return None

        def get_transport_setting_details(self):
            if (self.protocol.lower() == EConfigType.VMESS.name.lower() or
                    self.protocol.lower() == EConfigType.VLESS.name.lower() or
                    self.protocol.lower() == EConfigType.TROJAN.name.lower()):
                transport = self.streamSettings.network
                if transport == "tcp":
                    tcp_setting = self.streamSettings.tcpSettings
                    return [
                        tcp_setting.header.type,
                        ",".join(tcp_setting.header.request.headers.Host) if tcp_setting.header.request else "",
                        ",".join(tcp_setting.header.request.path) if tcp_setting.header.request else ""
                    ]
                elif transport == "kcp":
                    kcp_setting = self.streamSettings.kcpSettings
                    return [kcp_setting.header.type, "", kcp_setting.seed]
                elif transport == "ws":
                    ws_setting = self.streamSettings.wsSettings
                    return ["", ws_setting.headers.Host, ws_setting.path]
                elif transport == "httpupgrade":
                    httpupgrade_setting = self.streamSettings.httpupgradeSettings
                    return ["", httpupgrade_setting.host, httpupgrade_setting.path]
                elif transport == "xhttp":
                    xhttp_settings = self.streamSettings.xhttpSettings
                    return ["", xhttp_settings.host, xhttp_settings.path]
                elif transport == "h2":
                    h2_setting = self.streamSettings.httpSettings
                    return ["", ",".join(h2_setting.host), h2_setting.path]
                elif transport == "quic":
                    quic_setting = self.streamSettings.quicSettings
                    return [quic_setting.header.type, quic_setting.security, quic_setting.key]
                elif transport == "grpc":
                    grpc_setting = self.streamSettings.grpcSettings
                    return ["multi" if grpc_setting.multiMode else "gun", "", grpc_setting.serviceName]
            return None

    class DnsBean(BaseConfig):
        def __init__(self, servers=None, hosts=None, client_ip=None,
                     disable_cache=None, query_strategy=None, tag=None):
            self.servers = servers or []
            self.hosts = hosts or {}
            self.clientIp = client_ip
            self.disableCache = disable_cache
            self.queryStrategy = query_strategy
            self.tag = tag

        class ServersBean(BaseConfig):
            def __init__(self, address="", port=None, domains=None, expect_i_ps=None,
                         client_ip=None):
                self.address = address
                self.port = port
                self.domains = domains or []
                self.expectIPs = expect_i_ps or []
                self.clientIp = client_ip

    class RoutingBean(BaseConfig):
        def __init__(self, domain_strategy, domain_matcher=None, rules=None, balancers=None):
            self.domainStrategy = domain_strategy
            self.domainMatcher = domain_matcher
            self.rules = rules or []
            self.balancers = balancers or []

        class RulesBean(BaseConfig):
            def __init__(self, type="", ip=None, domain=None, outbound_tag="",
                         balancer_tag=None, port=None, source_port=None,
                         network=None, source=None, user=None, inbound_tag=None,
                         protocol=None, attrs=None, domain_matcher=None):
                self.type = type
                self.ip = ip or []
                self.domain = domain or []
                self.outboundTag = outbound_tag
                self.balancerTag = balancer_tag
                self.port = port
                self.sourcePort = source_port
                self.network = network
                self.source = source or []
                self.user = user or []
                self.inboundTag = inbound_tag or []
                self.protocol = protocol or []
                self.attrs = attrs
                self.domainMatcher = domain_matcher

    class PolicyBean(BaseConfig):
        def __init__(self, levels=None, system=None):
            self.levels = levels or {}
            self.system = system

        class LevelBean(BaseConfig):
            def __init__(self, handshake=None, conn_idle=None, uplink_only=None,
                         downlink_only=None, stats_user_uplink=None, stats_user_downlink=None,
                         buffer_size=None):
                self.handshake = handshake
                self.connIdle = conn_idle
                self.uplinkOnly = uplink_only
                self.downlinkOnly = downlink_only
                self.statsUserUplink = stats_user_uplink
                self.statsUserDownlink = stats_user_downlink
                self.bufferSize = buffer_size

    class FakednsBean(BaseConfig):
        def __init__(self, ip_pool="198.18.0.0/15", pool_size=10000):
            self.ipPool = ip_pool
            self.poolSize = pool_size

    def get_proxy_outbound(self):
        for outbound in self.outbounds:
            for protocol in EConfigType:
                if outbound.protocol.lower() == protocol.name.lower():
                    return outbound
        return None

    def to_pretty_printing(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)



import base64
import json
import logging
from typing import Optional


class Base64Helper:
    BOLLOCKS_LENGTH = 4

    @classmethod
    def fix_length(cls, encoded_base64_data):
        return encoded_base64_data + (len(encoded_base64_data) % cls.BOLLOCKS_LENGTH) * "="

    @classmethod
    def decode_base64(cls, encoded_base64_data) -> Optional[str]:
        """
            Decode base64
        """
        try:
            return base64.urlsafe_b64decode(cls.fix_length(encoded_base64_data=encoded_base64_data)).decode()
        except Exception as e:
            logging.error(f"error in decode `{encoded_base64_data}`, e: {e}")
            return None

    @classmethod
    def convert_json_to_dict(cls, jsonify_dict_data):
        try:
            result = json.loads(jsonify_dict_data)
            if type(result) == dict:
                return result
        except (json.JSONDecodeError, UnicodeDecodeError) as e:
            logging.error(f"error in decode json `{jsonify_dict_data}`, e: {e}")

    @classmethod
    def decode_dict_base64(cls, encoded_dict_data) -> Optional[dict]:
        """
            Get dict from base64
        """
        jsonify_dict_data = cls.decode_base64(encoded_base64_data=encoded_dict_data)
        if not jsonify_dict_data:
            return
        return cls.convert_json_to_dict(jsonify_dict_data=jsonify_dict_data)

    @classmethod
    def encode_base64(cls, str_data: str) -> str:
        """
            Convert str to base64
        """
        return base64.urlsafe_b64encode(str_data.encode("utf-8")).decode("utf-8")

    @classmethod
    def encode_dict_base64(cls, dict_data: dict) -> str:
        """
            Convert dict to base64
        """
        jsonify_dict_data = json.dumps(dict_data)
        return cls.encode_base64(str_data=jsonify_dict_data)



TAG_AGENT = "proxy"
TAG_DIRECT = "direct"
TAG_BLOCKED = "block"

DNS_AGENT = "1.1.1.1"
DNS_DIRECT = "223.5.5.5"

HTTP_PROTOCOL = "http://"
HTTPS_PROTOCOL = "https://"

PREF_MODE = "pref_mode"
PREF_SPEED_ENABLED = True
PREF_SNIFFING_ENABLED = "pref_sniffing_enabled"
PREF_PROXY_SHARING = False
PREF_LOCAL_DNS_ENABLED = False
PREF_FAKE_DNS_ENABLED = "pref_fake_dns_enabled"
PREF_VPN_DNS = "pref_vpn_dns"
PREF_REMOTE_DNS = "pref_remote_dns"
PREF_DOMESTIC_DNS = "pref_domestic_dns"
PREF_LOCAL_DNS_PORT = "pref_local_dns_port"
PREF_ALLOW_INSECURE = "pref_allow_insecure"
PREF_SOCKS_PORT = "pref_socks_port"
PREF_HTTP_PORT = "pref_http_port"
PREF_LOGLEVEL = "info"
PREF_LANGUAGE = "pref_language"
PREF_PREFER_IPV6 = "pref_prefer_ipv6"
PREF_ROUTING_DOMAIN_STRATEGY = "pref_routing_domain_strategy"
PREF_ROUTING_MODE = "pref_routing_mode"
PREF_V2RAY_ROUTING_AGENT = "pref_v2ray_routing_agent"
PREF_V2RAY_ROUTING_DIRECT = "pref_v2ray_routing_direct"
PREF_V2RAY_ROUTING_BLOCKED = "pref_v2ray_routing_blocked"
PREF_PER_APP_PROXY = "pref_per_app_proxy"
PREF_PER_APP_PROXY_SET = "pref_per_app_proxy_set"
PREF_BYPASS_APPS = "pref_bypass_apps"
PREF_CONFIRM_REMOVE = "pref_confirm_remove"
PREF_START_SCAN_IMMEDIATE = "pref_start_scan_immediate"

PORT_LOCAL_DNS = 10853
PORT_SOCKS = 10808
PORT_HTTP = 10809



BASE_XRAY_CONFIG = {
    "stats": {},
    "log": {"loglevel": "warning"},
    "policy": {
        "levels": {"8": {"handshake": 4, "connIdle": 300, "uplinkOnly": 1, "downlinkOnly": 1}},
        "system": {"statsOutboundUplink": True, "statsOutboundDownlink": True}
    },
    "inbounds": [
        {"tag": "socks", "port": 10808, "protocol": "socks",
         "settings": {"auth": "noauth", "udp": True, "userLevel": 8},
         "sniffing": {"enabled": True, "destOverride": ["http", "tls"]}},
        {"tag": "http", "port": 10809, "protocol": "http", "settings": {"userLevel": 8}}
    ],
    "outbounds": [
        {"tag": "proxy", "protocol": "vmess", "settings": {"vnext": [{"address": "v2ray.cool", "port": 10086, "users": [
            {"id": "a3482e88-686a-4a58-8126-99c9df64b7bf", "alterId": 0, "security": "auto", "level": 8}]}],
                                                           "servers": [{"address": "v2ray.cool", "method": "chacha20",
                                                                        "ota": False, "password": "123456",
                                                                        "port": 10086, "level": 8}]},
         "streamSettings": {"network": "tcp"}, "mux": {"enabled": False}},
        {"protocol": "freedom", "settings": {}, "tag": "direct"},
        {"protocol": "blackhole", "tag": "block", "settings": {"response": {"type": "http"}}}
    ],
    "routing": {"domainStrategy": "IPIfNonMatch", "rules": []},
    "dns": {"hosts": {}, "servers": []}
}



import base64
import logging
import re
from typing import List
from urllib.parse import unquote


class Utils:
    @staticmethod
    def decode(text: str) -> str:
        try:
            return base64.urlsafe_b64decode(text + (len(text) % 4) * "=").decode()
        except Exception as e:
            # logging.error(f"can not decode {text}. e: {e}")
            return ""

    @staticmethod
    def get_remote_dns_servers() -> List[str]:
        ret = ['1.1.1.1']
        return ret

    """
    @staticmethod
    def get_vpn_dns_servers() -> List[str]:
        vpn_dns = (settingsStorage.decodeString(AppConfig.PREF_VPN_DNS) or settingsStorage.decodeString(
            AppConfig.PREF_REMOTE_DNS) or AppConfig.DNS_AGENT)
        return [server for server in vpn_dns.split(",") if Utils.is_pure_ip_address(server)]

    @staticmethod
    def get_domestic_dns_servers() -> List[str]:
        domestic_dns = settingsStorage.decodeString(AppConfig.PREF_DOMESTIC_DNS) or AppConfig.DNS_DIRECT
        ret = [server for server in domestic_dns.split(",") if
               Utils.is_pure_ip_address(server) or Utils.is_core_dns_address(server)]
        if not ret:
            return [AppConfig.DNS_DIRECT]
        return ret

    @staticmethod
    def is_ip_address(value: str) -> bool:
        try:
            addr = value
            if not addr or addr.isspace():
                return False
            if '/' in addr:
                arr = addr.split("/")
                if len(arr) == 2 and int(arr[1]) > 0:
                    addr = arr[0]
            if addr.startswith("::ffff:") and '.' in addr:
                addr = addr[7:]
            elif addr.startswith("[::ffff:") and '.' in addr:
                addr = addr[8:].replace("]", "")
            octets = addr.split('.')
            if len(octets) == 4:
                if ':' in octets[3]:
                    addr = addr[:addr.index(":")]
                return Utils.is_ipv4_address(addr)
            return Utils.is_ipv6_address(addr)
        except Exception as e:
            print(e)
            return False
    """

    @staticmethod
    def is_pure_ip_address(value: str) -> bool:
        return Utils.is_ipv4_address(value) or Utils.is_ipv6_address(value)

    @staticmethod
    def is_ipv4_address(value: str) -> bool:
        reg_v4 = re.compile(r"^([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])\."
                            r"([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])\."
                            r"([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])\."
                            r"([01]?[0-9]?[0-9]|2[0-4][0-9]|25[0-5])$")
        return bool(reg_v4.match(value))

    @staticmethod
    def is_ipv6_address(value: str) -> bool:
        addr = value
        if addr.startswith("[") and addr.rfind("]") > 0:
            addr = addr[1:addr.rfind("]")]
        reg_v6 = re.compile(r"^([0-9A-Fa-f]{1,4})?"
                            r"(:[0-9A-Fa-f]{1,4})*::"
                            r"([0-9A-Fa-f]{1,4})?"
                            r"(:[0-9A-Fa-f]{1,4})*"
                            r"|([0-9A-Fa-f]{1,4})"
                            r"(:[0-9A-Fa-f]{1,4}){7}$")
        return bool(reg_v6.match(addr))

    """
    @staticmethod
    def is_core_dns_address(s: str) -> bool:
        return s.startswith("https") or s.startswith("tcp") or s.startswith("quic")


    @staticmethod
    def is_valid_url(value: str) -> bool:
        try:
            if (re.match(Patterns.WEB_URL.pattern, value) or URLUtil.isValidUrl(value)):
                return True
        except Exception as e:
            print(e)
            return False
        return False
    """

    @staticmethod
    def url_decode(url) -> str:
        try:
            return unquote(url)
        except Exception as e:
            # logging.error(e)
            # logging.exception(e)
            return url

    @staticmethod
    def read_text_from_assets(file_name) -> str:
        with open(file_name) as file:
            return file.read()

    """
    @staticmethod
    def get_ipv6_address(address) -> str:
        if Utils.is_ipv6_address(address):
            return f"[{address}]"
        return address


    @staticmethod
    def fix_illegal_url(s) -> str:
        return s.replace(" ", "%20").replace("|", "%7C")


    @staticmethod
    def remove_white_space(str_) -> Union[str, None]:
        return str_.replace(" ", "") if str_ else None

    @staticmethod
    def idn_to_ascii(str_) -> str:
        url = urlparse(str_)
        return urlunparse(
           (url.scheme, IDN.toASCII(url.netloc, IDN.ALLOW_UNASSIGNED), url.path, url.params, url.query, url.fragment))

    @staticmethod
    def is_tv(context) -> bool:
        return context.packageManager.hasSystemFeature(PackageManager.FEATURE_LEANBACK)
    """



from enum import Enum


class EConfigType(Enum):
    VMESS = ("VMESS", "vmess://")
    CUSTOM = ("CUSTOM", "")
    SHADOWSOCKS = ("SHADOWSOCKS", "ss://")
    SOCKS = ("SOCKS", "socks://")
    VLESS = ("VLESS", "vless://")
    TROJAN = ("TROJAN", "trojan://")
    WIREGUARD = ("WIREGUARD", "wireguard://")
